package edu.mermet.tp8;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import edu.mermet.tp8.fenetres.FenetreBoutons;
import edu.mermet.tp8.fenetres.FenetreConversion;
import edu.mermet.tp8.fenetres.FenetreDiaporama;
import edu.mermet.tp8.fenetres.FenetreTexte;

/**
 *
 * @author brunomermet
 */
public class Application extends JFrame implements PropertyChangeListener {
    private JInternalFrame conversion;
    private JInternalFrame texte;
    private JInternalFrame diaporama;
    private JInternalFrame boutons;
    private Action actionAfficherConversion;
    private Action actionAfficherTexte;
    private Action actionAfficherDiaporama;
    private Action actionAfficherBoutons;
    private PropertyChangeSupport gestionEcouteurs;
    public Application() {
        super("multi-fenêtres");
        this.setContentPane(new JDesktopPane());

        // ****** Barre de menu ******
        JMenuBar barre = new JMenuBar();
        // ------ menu Fichier ------
        JMenu menuFichier = new JMenu("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F);
        JMenuItem quitter = new JMenuItem("Quitter");
        quitter.addActionListener (new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent aev) {
                System.exit(0);
            }
        });
        quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        menuFichier.add(quitter);
        barre.add(menuFichier);
        this.setJMenuBar(barre);
        // ------ menu Applications ------
        JMenu menuApplication = new JMenu("Applications");
        menuApplication.setMnemonic(KeyEvent.VK_A);
        actionAfficherConversion = new ActionAfficherConversion();
        JMenuItem itemConversion = new JMenuItem(actionAfficherConversion);
        menuApplication.add(itemConversion);
        actionAfficherTexte = new ActionAfficherTexte();
        JMenuItem itemTexte = new JMenuItem(actionAfficherTexte);
        menuApplication.add(itemTexte);
        actionAfficherDiaporama = new ActionAfficherDiaporama();
        JMenuItem itemDiaporama = new JMenuItem(actionAfficherDiaporama);
        menuApplication.add(itemDiaporama);
        actionAfficherBoutons = new ActionAfficherBoutons();
        JMenuItem itemBoutons = new JMenuItem(actionAfficherBoutons);
        menuApplication.add(itemBoutons);
        barre.add(menuApplication);
        // ------ menu Langues ------
        JMenu menuLangues = new JMenu("Langues");
        ButtonGroup groupe = new ButtonGroup();
        menuLangues.setMnemonic(KeyEvent.VK_L);
        int numBouton = 0;
        
        ActionChangerLangue action = new ActionChangerLangue(this, numBouton++, "Par dfaut", true);
        JRadioButton btnLangueDefaut = new JRadioButton(action);
        groupe.add(btnLangueDefaut);
        menuLangues.add(btnLangueDefaut);

        action = new ActionChangerLangue(this, numBouton++, "Anglais", false);
        JRadioButton btnLangueAnglais = new JRadioButton(action);
        groupe.add(btnLangueAnglais);
        menuLangues.add(btnLangueAnglais);

        action = new ActionChangerLangue(this, numBouton++, "Espagnol", false);
        JRadioButton btnLangueEspagnol = new JRadioButton(action);
        groupe.add(btnLangueEspagnol);
        menuLangues.add(btnLangueEspagnol);

        action = new ActionChangerLangue(this, numBouton++, "Francais", false);
        JRadioButton btnLangueFrancais = new JRadioButton(action);
        groupe.add(btnLangueFrancais);
        menuLangues.add(btnLangueFrancais);
        
        barre.add(menuLangues);
        // ****** Fin barre de menu ******
        
        // ****** Création des fenêtres ******
        // ------ fenêtre conversion ------
        conversion = new FenetreConversion(actionAfficherConversion);
        this.add(conversion);
        // ------ fenêtre texte ------
        texte = new FenetreTexte(actionAfficherTexte);
        this.add(texte);
        // ------ fenêtre diaporama ------
        diaporama = new FenetreDiaporama(actionAfficherDiaporama);
        this.add(diaporama);
        // ------ fenêtre boutons ------
        boutons = new FenetreBoutons(this,actionAfficherBoutons);
        this.add(boutons);
        // ****** Fin création fenêtres ******
        
        // ****** Gestion des couteurs ******
        gestionEcouteurs = new PropertyChangeSupport(this);
        addPropertyChangeListener(this);
        addPropertyChangeListener((PropertyChangeListener) conversion);
        addPropertyChangeListener((PropertyChangeListener) texte);
        addPropertyChangeListener((PropertyChangeListener) diaporama);
        addPropertyChangeListener((PropertyChangeListener) boutons);
           
        // ****** Finalisation ******
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(600,300);
        this.setLocationRelativeTo(null);
        setVisible(true);
    }

    private class ActionAfficherBoutons extends AbstractAction {
        public ActionAfficherBoutons() {
            super("Boutons");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_B);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            boutons.setVisible(true);
            enableBoutons(false);
        }
    }

    private class ActionAfficherDiaporama extends AbstractAction {
        public ActionAfficherDiaporama() {
            super("Diaporama");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_D);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            diaporama.setVisible(true);
            enableDiaporama(false);
        }
    }

    private class ActionAfficherTexte extends AbstractAction {
        public ActionAfficherTexte() {
            super("Saisie de texte");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            texte.setVisible(true);
            enableTexte(false);
        }
    }
    
    private class ActionAfficherConversion extends AbstractAction {
        public ActionAfficherConversion() {
            super("Conversion Celsius/Farenheit");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            conversion.setVisible(true);
            enableConversion(false);
        }
    }
    
    private class ActionChangerLangue extends AbstractAction {
    	private Application app;
    	private int numBouton;
        public ActionChangerLangue(Application app, int numBouton, String nomBouton, boolean selected) {
        	super(nomBouton);
        	putValue(Action.SELECTED_KEY,selected);
        	this.app = app;
        	this.numBouton = numBouton;
        }

		@Override
		public void actionPerformed(ActionEvent ae) {
			String langue;
			switch(numBouton) {
				case 1 : langue = "en"; break;
				case 2 : langue = "es"; break;
				default : langue = "fr";
			}
			app.firePropertyChange(
				new PropertyChangeEvent(app, "langue", null, langue)
			);
		}
    }

    public void enableConversion(boolean b) {
        actionAfficherConversion.setEnabled(b);
    }

    public void enableTexte(boolean b) {
        actionAfficherTexte.setEnabled(b);
    }

    public void enableDiaporama(boolean b) {
        actionAfficherDiaporama.setEnabled(b);
    }

    public void enableBoutons(boolean b) {
        actionAfficherBoutons.setEnabled(b);
    }

    public Action getActionAfficherConversion() {
        return actionAfficherConversion;
    }

    public Action getActionAfficherTexte() {
        return actionAfficherTexte;
    }

    public Action getActionAfficherDiaporama() {
        return actionAfficherDiaporama;
    }

	@Override
	public void propertyChange(PropertyChangeEvent pce) {
		Locale lieu = new Locale((String)pce.getNewValue());
		ResourceBundle ressources = ResourceBundle.getBundle("textes", lieu);
		
		// titre
		setTitle(ressources.getString("TitreFenetre"));
		// Menu
		JMenuBar menu = this.getJMenuBar();
		for(int i=0; i<menu.getMenuCount(); i++) {
			menu.getMenu(i).setText(ressources.getString("Menu-"+i));
			if(i==0)
				menu.getMenu(i).getItem(0).setText(ressources.getString("Quitter"));
			if(i==1) {
				menu.getMenu(i).getItem(0).setText(ressources.getString("Conversion"));
				menu.getMenu(i).getItem(1).setText(ressources.getString("Saisie"));
				menu.getMenu(i).getItem(2).setText(ressources.getString("Diaporama"));
				menu.getMenu(i).getItem(3).setText(ressources.getString("Boutons"));
			}
			if(i==menu.getMenuCount()-1) {
				for(int j=0; j<menu.getMenu(i).getItemCount(); j++) {
					((JRadioButton) menu.getMenu(i).getMenuComponent(j)).setText(
						ressources.getString("Langue-"+j)
					);
				}
			}
		}
	}

    public void addPropertyChangeListener(PropertyChangeListener pcl) {
    	gestionEcouteurs.addPropertyChangeListener(pcl);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener pcl) {
    	gestionEcouteurs.removePropertyChangeListener(pcl);
    }
    
	public void firePropertyChange(PropertyChangeEvent pce) {
    	gestionEcouteurs.firePropertyChange(pce);
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(Application::new);
    }
}
